==> kv5500-0.1.tar.gz: kv5500-0.1/src/kv5500.cpp <==
/*****************************************************************************
 *
 *  kv5500 - utility library for kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <cstdio>
#include <cstring>

#include "kv5500.hpp"

using namespace KV5500;

const char* UsbNoBus::what() const throw()
{
    return "No bus found ( Check access permission ).";
}

const char* UsbNoDev::what() const throw()
{
    return "No device found";
}

const char* UsbOpen::what() const throw()
{
    return "Usb open error";
}

Scanner::~Scanner()
{}

Usb::Usb()
{
    usb_init();

    if(usb_find_busses() <= 0)
        throw UsbNoBus();

    if(usb_find_devices() <= 0)
        throw UsbNoDev();
}

void Usb::Scan(Scanner& obj)
{
    usb_bus* bus = 0;
    for (bus = usb_busses; bus; bus = bus->next) {

        if (bus->root_dev)
            DoScan(bus->root_dev, obj);
        else {

            struct usb_device* dev = 0;

            for (dev = bus->devices; dev; dev = dev->next)
                DoScan(dev, obj);
        }
    }
}

void Usb::DoScan(struct usb_device* dev, Scanner& obj)
{
    assert(dev);

    ScanChildren scan(dev, obj);

    obj.Scan(dev);
}

/// fe 50 da bb 7e          Wake up magic string?.. anyway it works.
bool KV5500::WakeUp(UsbDev& udev)
{
    char buf[5] = { 0xfe, 0x50, 0xda, 0xbb, 0x7e };
    if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0)
        return false;

    char rbuf[1000];
    udev.BulkRead(rbuf, sizeof(rbuf), 5000);

    if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0)
        return false;

    udev.BulkRead(rbuf, sizeof(rbuf), 5000);

    char buf2[6] = { 0xfe, 0x64, 0x7d, 0x5d, 0xcc, 0x7e };
    if(udev.BulkWrite(buf2, sizeof(buf2), 5000) < 0)
        return false;

    udev.BulkRead(rbuf, sizeof(rbuf), 5000);

    return true;
}

const char* const KV5500::GetString(const char* const buf, const size_t maxEndPoint, const unsigned int offset, const size_t maxStrSize, const char endCh)
{
    assert(buf && maxEndPoint > 0 && maxStrSize > 0);

    const char* const endPoint = reinterpret_cast<const char*>(memchr(buf + offset, endCh, maxEndPoint));
    const size_t strSize = endPoint ? 
        endPoint - (buf + offset) : maxStrSize;

    char* const str = new char[strSize + 1];

    memcpy(str, buf + offset, strSize);
    str[strSize] = '\0';

    return str;
}

==> kv5500-0.2.tar.gz: kv5500-0.2/src/kv5500.cpp <==
/*****************************************************************************
 *
 *  kv5500 - utility library for kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <cstdio>
#include <cstring>

#include "kv5500.hpp"

using namespace KV5500;

const char* UsbNoBus::what() const throw()
{
    return "No bus found ( Check access permission ).";
}

const char* UsbNoDev::what() const throw()
{
    return "No device found";
}

const char* UsbOpen::what() const throw()
{
    return "Usb open error";
}

Scanner::~Scanner()
{}

Usb::Usb()
{
    usb_init();

    if(usb_find_busses() <= 0)
        throw UsbNoBus();

    if(usb_find_devices() <= 0)
        throw UsbNoDev();
}

void Usb::Scan(Scanner& obj)
{
    for (usb_bus* bus = usb_busses; bus; bus = bus->next) {

        if (bus->root_dev)
            DoScan(bus->root_dev, obj);
        else {

            for (struct usb_device* dev = bus->devices; dev; dev = dev->next)
                DoScan(dev, obj);
        }
    }
}

void Usb::DoScan(struct usb_device* dev, Scanner& obj)
{
    assert(dev);

    ScanChildren scan(dev, obj);

    obj.Scan(dev);
}

/// fe 50 da bb 7e          Wake up magic string?.. anyway it works.
bool KV5500::WakeUp(UsbDev& udev)
{
    char buf[5] = { 0xfe, 0x50, 0xda, 0xbb, 0x7e };
    if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0)
        return false;

    char rbuf[1000];
    udev.BulkRead(rbuf, sizeof(rbuf), 5000);

    if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0)
        return false;

    udev.BulkRead(rbuf, sizeof(rbuf), 5000);

    char buf2[6] = { 0xfe, 0x64, 0x7d, 0x5d, 0xcc, 0x7e };
    if(udev.BulkWrite(buf2, sizeof(buf2), 5000) < 0)
        return false;

    udev.BulkRead(rbuf, sizeof(rbuf), 5000);

    return true;
}

std::string KV5500::GetString(const char* const buf, const size_t maxEndPoint, const unsigned int offset, const size_t maxStrSize, const char endCh)
{
    assert(buf && maxEndPoint > 0 && maxStrSize > 0);

    const char* const endPoint = reinterpret_cast<const char*>(memchr(buf + offset, endCh, maxEndPoint));
    const size_t strSize = endPoint ? 
        endPoint - (buf + offset) : maxStrSize;

    return std::string(buf + offset, strSize);
}
